@@ -50,14 +50,17 @@ class TrainManager:
     """ Manages training loop, validations, learning rate scheduling
     and early stopping."""
 
-    def __init__(self, model: Model, config: dict) -> None:
+    def __init__(self, model: Model, config: dict,
+                 batch_class: Batch = Batch) -> None:
         """
         Creates a new TrainManager for a model, specified as in configuration.
 
         :param model: torch module defining the model
         :param config: dictionary containing the training configurations
+        :param batch_class: batch class to encapsulate the torch class
         """
         train_config = config["training"]
+        self.batch_class = batch_class
 
         # files for logging and storing
         self.model_dir = train_config["model_dir"]
@@ -126,7 +129,7 @@ class TrainManager:
         # eval options
         test_config = config["testing"]
         self.bpe_type = test_config.get("bpe_type", "subword-nmt")
-        self.sacrebleu = {"remove_whitespace": True, "tokenizer": "13a"}
+        self.sacrebleu = {"remove_whitespace": True, "tokenize": "13a"}
         if "sacrebleu" in config["testing"].keys():
             self.sacrebleu["remove_whitespace"] = test_config["sacrebleu"] \
                 .get("remove_whitespace", True)
@@ -166,7 +169,7 @@ class TrainManager:
         self.n_gpu = torch.cuda.device_count() if self.use_cuda else 0
         self.device = torch.device("cuda" if self.use_cuda else "cpu")
         if self.use_cuda:
-            self.model.cuda()
+            self.model.to(self.device)
 
         # fp16
         self.fp16 = train_config.get("fp16", False)
@@ -297,7 +300,7 @@ class TrainManager:
 
         # move parameters to cuda
         if self.use_cuda:
-            self.model.cuda()
+            self.model.to(self.device)
 
         # fp16
         if self.fp16 and model_checkpoint.get("amp_state", None) is not None:
@@ -373,8 +376,8 @@ class TrainManager:
 
             for i, batch in enumerate(iter(train_iter)):
                 # create a Batch object from torchtext batch
-                batch = Batch(batch, self.model.pad_index,
-                              use_cuda=self.use_cuda)
+                batch = self.batch_class(batch, self.model.pad_index,
+                                         use_cuda=self.use_cuda)
 
                 # get batch loss
                 batch_loss += self._train_step(batch)
@@ -457,14 +460,11 @@ class TrainManager:
         self.model.train()
 
         # get loss
-        batch_loss, _, _, _ = self.model(
-            return_type="loss", src=batch.src, trg=batch.trg,
-            trg_input=batch.trg_input, src_mask=batch.src_mask,
-            src_length=batch.src_length, trg_mask=batch.trg_mask)
+        batch_loss, _, _, _ = self.model(return_type="loss", **vars(batch))
 
-        # average on multi-gpu parallel training
+        # sum multi-gpu losses
         if self.n_gpu > 1:
-            batch_loss = batch_loss.mean()
+            batch_loss = batch_loss.sum()
 
         # normalize batch loss
         if self.normalization == "batch":
@@ -480,6 +480,9 @@ class TrainManager:
 
         norm_batch_loss = batch_loss / normalizer
 
+        if self.n_gpu > 1:
+            norm_batch_loss = norm_batch_loss / self.n_gpu
+
         if self.batch_multiplier > 1:
             norm_batch_loss = norm_batch_loss / self.batch_multiplier
 
@@ -503,6 +506,7 @@ class TrainManager:
         valid_hypotheses_raw, valid_attention_scores = \
             validate_on_data(
                 batch_size=self.eval_batch_size,
+                batch_class=self.batch_class,
                 data=valid_data,
                 eval_metric=self.eval_metric,
                 level=self.level, model=self.model,
